@@ -14,17 +14,19 @@ sub new {
 
 sub _log {
     my ($self, $paths, $rev, $author, $date, $msg, $pool) = @_;
-    $self->{REV} = {rev => $rev, author => $author,
-		     date => $date, msg => $msg};
-    $self->{REV}{paths} = {map { $_ => {action => $paths->{$_}->action,
-					 copyfrom => $paths->{$_}->copyfrom_path,
-					 copyfromrev => $paths->{$_}->copyfrom_rev,
-				       }} keys %$paths};
+    my $data = {rev => $rev, author => $author,
+		date => $date, msg => $msg};
+
+    $data->{paths} = {map { $_ => {action => $paths->{$_}->action,
+				   copyfrom => $paths->{$_}->copyfrom_path,
+				   copyfromrev => $paths->{$_}->copyfrom_rev,
+				  }} keys %$paths};
     my $root = $self->{repos}->fs->revision_root ($rev, $pool);
-    for (keys %{$self->{REV}{paths}}) {
-	$self->{REV}{paths}{$_}{isdir} = 1
+    for (keys %{$data->{paths}}) {
+	$data->{paths}{$_}{isdir} = 1
 	    if SVN::Fs::check_path ($root, $_) == $SVN::Core::node_dir;
     }
+    return $data;
 }
 
 sub run {
@@ -33,7 +35,7 @@ sub run {
     my $rev = $self->{cgi}->param('rev') || die 'no revision';
 
     $self->{repos}->get_logs ([], $rev, $rev, 1, 0,
-			      sub { $self->_log(@_)});
+			      sub { $self->{REV} = $self->_log(@_)});
     return {template => 'revision',
 	    data => { rev => $rev, %{$self->{REV}}}};
 }
@@ -51,9 +53,20 @@ revision [% rev %] - [% author || '(no author)' %] - [% date %]:<br />
 [% path.value.action %] -
 [% IF path.value.isdir %]
 <a href="[% script %]/[% repos %]/browse[% path.key %]?rev=[% rev %]">[% path.key %]</a>
+[% IF path.value.copyfrom %]
+(from
+<a href="[% script %]/[% repos %]/browse[% path.value.copyfrom %]/?rev=[% path.value.copyfromrev %]">[% path.value.copyfrom %]:[% path.value.copyfromrev %]</a>
+)
+[% END %]
+
 [% ELSE %]
 <a href="[% script %]/[% repos %]/log[% path.key %]#rev[% rev %]">[% path.key %]</a>
 <a href="[% script %]/[% repos %]/checkout[% path.key %]?rev=[% rev %]">(checkout)</a>
+[% IF path.value.copyfrom %]
+(from
+<a href="[% script %]/[% repos %]/log[% path.value.copyfrom %]#rev[% path.value.copyfromrev %]">[% path.value.copyfrom %]:[% path.value.copyfromrev %]</a>
+)
+[% END %]
 [% END %]
 
 <br />
